@@ -6,7 +6,7 @@
 #
 # This program is licensed under Gnu General Public License version 2.
 
-CROSSER_MAINDIR=$(cd $(dirname $0) ; pwd)
+CROSSER_MAINDIR="$(cd "$(dirname "$0")" ; pwd)"
 
 if ! test -e "$CROSSER_MAINDIR/CrosserVersion"
 then
@@ -14,36 +14,36 @@ then
 fi
 
 STEP="setup"
-STEPADD="   "
+STEPADD="    "
 
 if test "x$1" = "x-h" || test "x$1" = "x--help" ; then
-  echo "Usage: $(basename $0) [target setup name] [steps] [versionset]"
+  echo "Usage: $(basename "$0") [target setup name] [steps] [versionset]"
   exit 0
 fi
 
 # In order to give local setup opportunity to override versions,
-# we have to load versionset before build_setup.conf
-# helpers.sh requires environment to be set up by build_setup.conf.
+# we have to load versionset before setup_reader.sh
+# helpers.sh requires environment to be set up by setup_reader.sh.
 if test "x$3" != "x" ; then
   VERSIONSET="$3"
 else
   VERSIONSET="current"
 fi
-if test -e $CROSSER_MAINDIR/setups/$VERSIONSET.versions
+if test -e "$CROSSER_MAINDIR/setups/$VERSIONSET.versions"
 then
-  . $CROSSER_MAINDIR/setups/$VERSIONSET.versions
+  . "$CROSSER_MAINDIR/setups/$VERSIONSET.versions"
 else
-  # Versions being unset do not prevent loading of build_setup.conf and helper.sh,
+  # Versions being unset do not prevent loading of setup_reader.sh and helper.sh,
   # resulting environment would just be unusable for building.
   # We are not going to build anything, but just issuing error message - and for
   # that we read log_error from helpers.sh
   ERR_MSG="Cannot find versionset \"$VERSIONSET.versions\""
 fi
 
-. $CROSSER_MAINDIR/build_setup.conf
-. $CROSSER_MAINDIR/scripts/helpers.sh
-. $CROSSER_MAINDIR/scripts/packethandlers.sh
-. $CROSSER_MAINDIR/scripts/buildfuncs.sh
+. "$CROSSER_MAINDIR/scripts/setup_reader.sh"
+. "$CROSSER_MAINDIR/scripts/helpers.sh"
+. "$CROSSER_MAINDIR/scripts/packethandlers.sh"
+. "$CROSSER_MAINDIR/scripts/buildfuncs.sh"
 
 # This must be after reading helpers.sh so that $CROSSER_VERSION is set
 if test "x$1" = "x-v" || test "x$1" = "x--version"
@@ -106,7 +106,7 @@ else
   STEPPARAM="native:sdl"
 fi
 
-. $CROSSER_MAINDIR/steps/stepfuncs.sh
+. "$CROSSER_MAINDIR/steps/stepfuncs.sh"
 
 STEPLIST="$(parse_steplist $STEPPARAM)"
 
@@ -117,13 +117,13 @@ then
 fi
 
 # Setup step variables
-. $CROSSER_MAINDIR/steps/stepset.sh
+. "$CROSSER_MAINDIR/steps/stepset.sh"
 
 if ! test -e "$CROSSER_MAINDIR/setups/$SETUP.conf" ; then
   log_error "Can't find setup \"$SETUP.conf\""
   exit 1
 fi
-source "$CROSSER_MAINDIR/setups/$SETUP.conf"
+. "$CROSSER_MAINDIR/setups/$SETUP.conf"
 
 if test "x$LIBC_MODE" = "xnone"
 then
@@ -138,7 +138,7 @@ else
   TARGET="$TARGET_ARCH-$TARGET_VENDOR-$TARGET_OS"
 fi
 
-if ! source $CROSSER_MAINDIR/setups/native.sh ; then
+if ! . "$CROSSER_MAINDIR/setups/native.sh" ; then
   log_error "Failed to read $CROSSER_MAINDIR/setups/native.sh"
   exit 1
 fi
@@ -153,7 +153,7 @@ else
   BUILD="$NATIVE_ARCH-$NATIVE_VENDOR-$NATIVE_OS"
 fi
 
-NATIVE_PREFIX=$(setup_prefix_default "$HOME/.crosser/$CROSSER_VERSION/host" "$NATIVE_PREFIX")
+NATIVE_PREFIX=$(setup_prefix_default "$HOME/.crosser/<VERSION>/<VERSIONSET>/host" "$NATIVE_PREFIX")
 
 CH_ERROR="$(check_crosser_env $NATIVE_PREFIX native)"
 if test "x$CH_ERROR" != "x" && test "x$STEP_NATIVE" != "xyes" ; then
@@ -165,21 +165,23 @@ fi
 # If native build, and cross-compile is not already being forced
 if test "x$TARGET" = "x$BUILD" && test "x$CROSS_OFF" = "x" ; then
   CROSS_OFF=yes
-  DEFPREFIX="$HOME/.crosser/$CROSSER_VERSION/native"
+  DEFPREFIX="$HOME/.crosser/<VERSION>/<VERSIONSET>/native"
 else
-  DEFPREFIX="$HOME/.crosser/$CROSSER_VERSION/$TARGET-$LIBC_MODE"
+  DEFPREFIX="$HOME/.crosser/<VERSION>/<VERSIONSET>/<TARGET>-<CLIB>"
 fi
 
-PREFIX=$(setup_prefix_default "$DEFPREFIX" "$PREFIX")
-SYSPREFIX="$PREFIX/target"
-CROSSPREFIX="$PREFIX/crosstools"
+CROSSER_DST_PFX=$(setup_prefix_default "$DEFPREFIX" "$CROSSER_DST_PFX")
+SYSPREFIX="$CROSSER_DST_PFX/target"
+CROSSPREFIX="$CROSSER_DST_PFX/crosstools"
+CROSSER_IM_PFX="$CROSSER_DST_PFX/interm"
 
 if test "x$CROSS_OFF" != "xyes"
 then
-  CH_ERROR="$(check_crosser_env $PREFIX $TARGET)"
+  CH_ERROR="$(check_crosser_env $CROSSER_DST_PFX $TARGET)"
   if test "x$CH_ERROR" != "x" &&
      test "x$STEP_CHAIN" != "xyes" &&
      ( test "x$STEP_BASELIB" = "xyes" ||
+       test "x$STEP_XORG" = "xyes" ||
        test "x$STEP_GTK" = "xyes" ||
        test "x$STEP_SDL" = "xyes" )
   then
@@ -214,7 +216,7 @@ build_glibc() {
   then
     log_write 3 "  Copying dummy headers"
 
-    if ! cp "$MAINBUILDDIR/tgt-$1/bits/stdio_lim.h" \
+    if ! cp "$CROSSER_BUILDDIR/tgt-$1/bits/stdio_lim.h" \
             "$SYSPREFIX/usr/include/bits/"
     then
       log_error "Failed to copy initial stdio_lim.h"
@@ -269,7 +271,7 @@ create_host_dirs()
     return 1
   fi
 
-  if ! mkdir -p $NATIVE_PREFIX/usr/include
+  if ! mkdir -p "$NATIVE_PREFIX/usr/include"
   then
     log_error "Failed to create host directories under \"$NATIVE_PREFIX\""
     return 1
@@ -280,12 +282,12 @@ create_host_dirs()
 #
 create_target_dirs()
 {
-  if ! mkdir -p $SYSPREFIX/etc         ||
-     ! mkdir -p $SYSPREFIX/include     ||
-     ! mkdir -p $SYSPREFIX/usr/include ||
-     ! mkdir -p $SYSPREFIX/usr/lib
+  if ! mkdir -p "$SYSPREFIX/etc"         ||
+     ! mkdir -p "$SYSPREFIX/include"     ||
+     ! mkdir -p "$SYSPREFIX/usr/include" ||
+     ! mkdir -p "$SYSPREFIX/usr/lib"
   then
-    log_error "Failed to create target directories under \"$PREFIX\""
+    log_error "Failed to create target directories under \"$SYSPREFIX\""
     return 1
   fi
 }
@@ -305,7 +307,7 @@ kernel_setup() {
   fi
 
   # Kernel may have been unpacked already in chain step
-  if ! test -d $MAINSRCDIR/linux-$VERSION_KERNEL &&
+  if ! test -d "$CROSSER_SRCDIR/linux-$VERSION_KERNEL" &&
      ! unpack_component linux $VERSION_KERNEL
   then
     log_error "Failed to unpack kernel"
@@ -313,10 +315,10 @@ kernel_setup() {
   fi
 
   if ! (
-    cd $MAINSRCDIR/linux-$VERSION_KERNEL
+    cd "$CROSSER_SRCDIR/linux-$VERSION_KERNEL"
 
-    if test -f $PACKETDIR/kernel-$TARGET.config ; then
-      cp $PACKETDIR/kernel-$TARGET.config .config
+    if test -f "$PACKETDIR/kernel-$TARGET.config" ; then
+      cp "$PACKETDIR/kernel-$TARGET.config" .config
       KCONFTARGET=silentoldconfig
     else
       KCONFTARGET=defconfig
@@ -335,7 +337,7 @@ kernel_setup() {
     log_write 3 "  Make params: $MAKEPARAMS"
 
     if ! make $MAKEPARAMS \
-	           2>>$LOGDIR/stderr.log >>$LOGDIR/stdout.log
+	           2>> "$CROSSER_LOGDIR/stderr.log" >> "$CROSSER_LOGDIR/stdout.log"
     then
       log_error "Kernel prepare failed"
       return 1
@@ -352,7 +354,7 @@ kernel_setup() {
 
     log_write 3 "  Make params: $MAKEPARAMS"
     if ! make $MAKEPARAMS \
-                2>>$LOGDIR/stderr.log >>$LOGDIR/stdout.log
+                2>> "$CROSSER_LOGDIR/stderr.log" >> "$CROSSER_LOGDIR/stdout.log"
     then
       if test "x$1" = "xfull"
       then
@@ -396,7 +398,7 @@ link_host_command() {
     fi
   fi
 
-  if ! ln -s $CMDPATH $NATIVE_PREFIX/hostbin/
+  if ! ln -s "$CMDPATH" "$NATIVE_PREFIX/hostbin/"
   then
     log_error "Failed to make symbolic link to host command $1"
     return 1
@@ -409,14 +411,14 @@ link_host_command() {
 # 1 - Failure
 setup_host_commands() {
   # Absolutely required commands
-  HOST_COMMANDS_REQ="mkdir touch true false chmod rm which sed grep expr cat echo sort mv cp ln cmp test comm ls rmdir tr date uniq sleep diff basename dirname tail head env uname cut readlink od egrep fgrep wc make find pwd tar m4 awk getconf expand perl bison bzip2 flex makeinfo install whoami depmod wget pod2man msgfmt pkg-config sh glib-genmarshal hostname dnsdomainname mktemp"
+  HOST_COMMANDS_REQ="mkdir touch true false chmod rm which sed grep expr cat echo sort mv cp ln cmp test comm ls rmdir tr date uniq sleep diff basename dirname tail head env uname cut readlink od egrep fgrep wc make find pwd tar m4 awk getconf expand perl bison bzip2 flex makeinfo install whoami depmod wget pod2man msgfmt sh glib-genmarshal hostname dnsdomainname mktemp"
   # Usefull commands
   HOST_COMMANDS_TRY="dpkg-source md5sum gpg gpgv sha1sum sha256sum gzip gunzip patch quilt bash getopt"
 
   log_write 1 "Setting up hostbin"
 
-  if ! mkdir -p $NATIVE_PREFIX/hostbin ; then
-    log_error "Cannot create directory $PREFIX/hostbin"
+  if ! mkdir -p "$NATIVE_PREFIX/hostbin" ; then
+    log_error "Cannot create directory $NATIVE_PREFIX/hostbin"
     return 1
   fi
 
@@ -442,12 +444,12 @@ setup_host_commands() {
 dummy_glibc_objects() {
   log_write 1 "Generating dummy c-lib objects"
 
-  if ! mkdir $MAINBUILDDIR/crt ; then
+  if ! mkdir "$CROSSER_BUILDDIR/crt" ; then
      return 1
   fi
 
   if ! (
-    cd $MAINBUILDDIR/crt
+    cd "$CROSSER_BUILDDIR/crt"
 
     echo "/* Build dummy crt.o object from this comment */" > crt.c
 
@@ -455,30 +457,30 @@ dummy_glibc_objects() {
        log_error "crt.o build failed"
        return 1
     fi
-    if ! $TARGET-gcc -c -shared -fPIC $CROSSER_MAINDIR/scripts/dummyclib.c -o libc.so \
-         2>>$LOGDIR/stderr.log >>$LOGDIR/stdout.log
+    if ! $TARGET-gcc -c -shared -fPIC "$CROSSER_MAINDIR/scripts/dummyclib.c" -o libc.so \
+         2>> "$CROSSER_LOGDIR/stderr.log" >> "$CROSSER_LOGDIR/stdout.log"
     then
         log_error "Failed to build dummy libc.so"
        return 1
     fi
 
-    if ! test -e $SYSPREFIX/usr/lib/crt1.o &&
-       ! cp crt.o $SYSPREFIX/usr/lib/crt1.o ; then
+    if ! test -e "$SYSPREFIX/usr/lib/crt1.o" &&
+       ! cp crt.o "$SYSPREFIX/usr/lib/crt1.o" ; then
        log_error "Failed to copy crt1.o"
        return 1
     fi
-    if ! test -e $SYSPREFIX/usr/lib/crti.o &&
-       ! cp crt.o $SYSPREFIX/usr/lib/crti.o ; then
+    if ! test -e "$SYSPREFIX/usr/lib/crti.o" &&
+       ! cp crt.o "$SYSPREFIX/usr/lib/crti.o" ; then
        log_error "Failed to copy crti.o"
        return 1
     fi
-    if ! test -e $SYSPREFIX/usr/lib/crtn.o &&
-       ! cp crt.o $SYSPREFIX/usr/lib/crtn.o ; then
+    if ! test -e "$SYSPREFIX/usr/lib/crtn.o" &&
+       ! cp crt.o "$SYSPREFIX/usr/lib/crtn.o" ; then
        log_error "Failed to copy crtn.o"
        return 1
     fi
-    if ! test -e $SYSPREFIX/usr/lib/libc.so &&
-       ! cp libc.so $SYSPREFIX/usr/lib/libc.so ; then
+    if ! test -e "$SYSPREFIX/usr/lib/libc.so" &&
+       ! cp libc.so "$SYSPREFIX/usr/lib/libc.so" ; then
        log_error "Failed to copy libc.so"
        return 1
     fi
@@ -494,7 +496,7 @@ prepare_binutils_src() {
   if ! unpack_component binutils     $VERSION_BINUTILS              ||
      ! ( is_greater_version $VERSION_BINUTILS 2.18     ||
         (  patch_src binutils-$VERSION_BINUTILS binutils_makeinfo &&
-           cd $MAINSRCDIR/binutils-$VERSION_BINUTILS && autoconf )) ||
+           cd "$CROSSER_SRCDIR/binutils-$VERSION_BINUTILS" && autoconf )) ||
      ! ( is_greater_version $VERSION_BINUTILS 2.18     ||
         (  patch_src binutils-$VERSION_BINUTILS binutils_printffix &&
 	   patch_src binutils-$VERSION_BINUTILS binutils_ignret ))  ||
@@ -526,20 +528,38 @@ prepare_gcc_src() {
     exit 1
   fi
 
-  if ! ln -s ../mpfr-$VERSION_MPFR $MAINSRCDIR/gcc-$VERSION_GCC/mpfr ||
-     ! ln -s ../gmp-$VERSION_GMP $MAINSRCDIR/gcc-$VERSION_GCC/gmp ||
+  if ! ln -s ../mpfr-$VERSION_MPFR "$CROSSER_SRCDIR/gcc-$VERSION_GCC/mpfr" ||
+     ! ln -s ../gmp-$VERSION_GMP "$CROSSER_SRCDIR/gcc-$VERSION_GCC/gmp" ||
      ! (is_smaller_version $VERSION_GCC 4.5.0 ||
-        ln -s ../mpc-$VERSION_MPC $MAINSRCDIR/gcc-$VERSION_GCC/mpc)
+        ln -s ../mpc-$VERSION_MPC "$CROSSER_SRCDIR/gcc-$VERSION_GCC/mpc")
   then
     log_error "Creation of links to additional gcc modules failed"
     exit 1
   fi
 }
 
-if test "x$TARGET" = "x$BUILD" && test "x$CROSS_OFF" = "x"
-then
-  CROSS_OFF=yes
-fi
+# Create link to intermediate build program to intermediate hostbin directory
+#
+# $1 - Program to link
+link_im_host_prog() {
+  if ! mkdir -p "$CROSSER_IM_PFX/hostbin"
+  then
+    log_error "Failed to create directory for intermediate build host programs"
+    return 1
+  fi
+
+  if test -f "$CROSSER_IM_PFX/hostbin/$1"
+  then
+    # Exist already
+    return 0
+  fi
+
+  if ! ln -s "$CROSSER_IM_PFX/bin/$1" "$CROSSER_IM_PFX/hostbin"
+  then
+    log_error "Failed to link intermediate build host program $1"
+    return 1
+  fi
+}
 
 if test "x$CROSS_OFF" = "xyes"
 then
@@ -547,26 +567,26 @@ then
 fi
 
 log_write 2 "Native tools: \"$NATIVE_PREFIX\""
-log_write 2 "Toolchain:    \"$PREFIX\""
+log_write 2 "Toolchain:    \"$CROSSER_DST_PFX\""
 log_write 2 "Target:       \"$TARGET\""
-log_write 2 "Src:          \"$MAINSRCDIR\""
-log_write 2 "Log:          \"$LOGDIR\""
-log_write 2 "Build:        \"$MAINBUILDDIR\""
+log_write 2 "Src:          \"$CROSSER_SRCDIR\""
+log_write 2 "Log:          \"$CROSSER_LOGDIR\""
+log_write 2 "Build:        \"$CROSSER_BUILDDIR\""
 log_write 2 "Setup:        \"$SETUP\""
 log_write 2 "Versionset:   \"$VERSIONSET\""
 log_write 2 "Steps:        \"$STEPLIST\""
 log_write 2 "c-lib:        \"$LIBC_MODE\""
 
-if ! remove_dir "$MAINBUILDDIR" ||
-   ! remove_dir "$MAINSRCDIR"   ||
+if ! remove_dir "$CROSSER_BUILDDIR" ||
+   ! remove_dir "$CROSSER_SRCDIR"   ||
    ! (test "x$STEP_NATIVE" != "xyes" || remove_dir "$NATIVE_PREFIX") ||
-   ! (test "x$STEP_CHAIN"  != "xyes" || remove_dir "$PREFIX")
+   ! (test "x$STEP_CHAIN"  != "xyes" || remove_dir "$CROSSER_DST_PFX")
 then
   log_error "Failed to remove old directories"
   exit 1
 fi
 
-if ! mkdir -p $MAINSRCDIR || ! mkdir -p $MAINBUILDDIR
+if ! mkdir -p "$CROSSER_SRCDIR" || ! mkdir -p "$CROSSER_BUILDDIR"
 then
   log_error "Cannot create main directories"
   exit 1
@@ -585,7 +605,7 @@ fi
 
 if test "x$CROSSER_DOWNLOAD" = "xyes"
 then
-  if ! ( cd $PACKETDIR && $CROSSER_MAINDIR/scripts/download_packets.sh "$STEPPARAM" )
+  if ! ( cd "$PACKETDIR" && "$CROSSER_MAINDIR/scripts/download_packets.sh" "$STEPPARAM" )
   then
     log_error "Download failed"
     exit 1
@@ -594,7 +614,7 @@ fi
 
 if test "x$STEP_NATIVE" = "xyes" ; then
   STEP="native"
-  STEPADD="  "
+  STEPADD="   "
 
   BASEVER_LIBTOOL="$(basever_libtool $VERSION_LIBTOOL)"
 
@@ -616,7 +636,18 @@ if test "x$STEP_NATIVE" = "xyes" ; then
      ! build_for_host   gtk-doc  gtk-doc-$VERSION_GTK_DOC         \
        "--disable-scrollkeeper"                                   ||
      ! unpack_component pkg-config $VERSION_PKG_CONFIG            ||
+     ! (! cmp_versions $VERSION_PKG_CONFIG 0.25 ||
+        patch_src pkg-config-$VERSION_PKG_CONFIG pkgconfig_ac266) ||
      ! build_for_host   pkg-config pkg-config-$VERSION_PKG_CONFIG ||
+     ! unpack_component glib       $VERSION_GLIB                  ||
+     ! build_for_host   glib       glib-$VERSION_GLIB             ||
+     ! unpack_component          libgpg-error $VERSION_GPGERROR     ||
+     ! build_for_host   libgpg-error libgpg-error-$VERSION_GPGERROR ||
+     ! unpack_component          libgcrypt $VERSION_LIBGCRYPT       ||
+     ! build_for_host   libgcrypt libgcrypt-$VERSION_LIBGCRYPT      \
+         "--disable-asm"                                            ||
+     ! unpack_component libxslt    $VERSION_LIBXSLT               ||
+     ! build_for_host   libxslt    libxslt-$VERSION_LIBXSLT       ||
      ! prepare_binutils_src                                   ||
      ! build_for_host binutils binutils-$VERSION_BINUTILS     \
      "--with-tls --enable-stage1-languages=all"               ||
@@ -629,6 +660,12 @@ if test "x$STEP_NATIVE" = "xyes" ; then
      exit 1
   fi
 
+  if ! ln -s gcc "$NATIVE_PREFIX/bin/cc"
+  then
+    crosser_error "Cannot create cc link for host"
+    exit 1
+  fi
+
   if ! setup_host_commands
   then
     crosser_error "Cannot enable selected host commands"
@@ -642,22 +679,24 @@ fi
 # This limits risk of host system commands from 'leaking' to our environments.
 # Commands that are safe to use from host system are accessed through hostbin.
 PATH_NATIVE="$NATIVE_PREFIX/bin:$NATIVE_PREFIX/hostbin"
-PATH_CROSS="$CROSSPREFIX/bin:$PATH_NATIVE"
+PATH_CROSS="$CROSSPREFIX/bin:$CROSSER_IM_PFX/hostbin:$PATH_NATIVE"
 
 if test "x$CROSSER_CCACHE" != "x" ; then
   PATH_NATIVE="$CROSSER_CCACHE:$PATH_NATIVE"
   PATH_CROSS="$CROSSER_CCACHE:$PATH_CROSS"
 fi
 
-if ! create_target_dirs ; then
-  crosser_error "Failed to create target dirs"
-  exit 1
+if test "x$STEPLIST" != "xnative" ; then
+  if ! create_target_dirs ; then
+    crosser_error "Failed to create target dirs"
+    exit 1
+  fi
 fi
 
 if test "x$STEP_CHAIN" = "xyes" && test "x$CROSS_OFF" != "xyes"
 then
   STEP="chain(1)"
-  STEPADD=""
+  STEPADD=" "
 
   export CCACHE_DIR="$NATIVE_PREFIX/.ccache"
 
@@ -710,16 +749,16 @@ then
     fi
 
     if ! ln -s ../newlib-$VERSION_NEWLIB/newlib \
-               $MAINSRCDIR/gcc-$VERSION_GCC ||
+               "$CROSSER_SRCDIR/gcc-$VERSION_GCC" ||
        ! ln -s ../newlib-$VERSION_NEWLIB/libgloss \
-               $MAINSRCDIR/gcc-$VERSION_GCC
+               "$CROSSER_SRCDIR/gcc-$VERSION_GCC"
     then
       crosser_error "Creation of newlib links failed"
       exit 1
     fi
 
     log_write 1 "Copying initial newlib headers"
-    if ! cp -R "$MAINSRCDIR/newlib-$VERSION_NEWLIB/newlib/libc/include" \
+    if ! cp -R "$CROSSER_SRCDIR/newlib-$VERSION_NEWLIB/newlib/libc/include" \
                "$SYSPREFIX/"
     then
       crosser_error "Failed initial newlib headers copying."
@@ -738,7 +777,8 @@ then
   if test "x$E_GLIBC" = "xyes"
   then
 
-    # Initial cross-compiler                                                                      
+    # Initial cross-compiler
+
     if ! build_with_native_compiler gcc gcc-$VERSION_GCC \
         "--enable-languages=c --with-gnu-as --with-gnu-ld --with-tls --with-sysroot=$SYSPREFIX --disable-multilib" \
         "all-gcc install-gcc"
@@ -764,7 +804,7 @@ then
         crosser_error "$LIBCNAME unpacking failed"
         exit 1
       fi
-      if ! ln -s $LIBCNAME $MAINSRCDIR/$LIBCDIR
+      if ! ln -s $LIBCNAME "$CROSSER_SRCDIR/$LIBCDIR"
       then
         crosser_error "Creation of $LIBCNAME links failed"
         exit 1
@@ -788,7 +828,7 @@ then
         crosser_error "$LIBCNAME unpacking failed"
         exit 1
       fi
-      if ! ln -s $LIBCNAME-ports-$VERSION_GLIBC_PORTS $MAINSRCDIR/$LIBCDIR/ports
+      if ! ln -s $LIBCNAME-ports-$VERSION_GLIBC_PORTS "$CROSSER_SRCDIR/$LIBCDIR/ports"
       then
 	crosser_error "Creation of $LIBCNAME links failed"
         exit 1
@@ -865,23 +905,28 @@ then
 
   fi
 
-  write_crosser_env "$PREFIX" "$TARGET"
+  write_crosser_env "$CROSSER_DST_PFX" "$TARGET"
 
-  generate_setup_scripts $PREFIX
+  generate_setup_scripts "$CROSSER_DST_PFX"
 else # STEP_CHAIN
   # Set PATH only if it's not already correct to avoid unnecessary hash reset.
   export PATH="$PATH_CROSS"
   hash -r
 fi   # STEP_CHAIN
 
-export CCACHE_DIR="$PREFIX/.ccache"
+export CCACHE_DIR="$CROSSER_DST_PFX/.ccache"
 
-export PKG_CONFIG_LIBDIR="$SYSPREFIX/lib/pkgconfig:$SYSPREFIX/usr/lib/pkgconfig"
+export PKG_CONFIG_LIBDIR="$CROSSER_IM_PFX/lib/pkgconfig:$SYSPREFIX/lib/pkgconfig:$SYSPREFIX/usr/lib/pkgconfig"
+
+PYTHON_SUBDIR="python$(echo $VERSION_PYTHON | sed 's/\./ /g' |
+( read MAJ MIN PATCH ; echo $MAJ.$MIN ))"
+
+export PYTHONPATH="$CROSSER_IM_PFX/lib/$PYTHON_SUBDIR/site-packages:$CROSSER_IM_PFX/lib/$PYTHON_SUBDIR/site-packages/xcbgen"
 
 if test "x$STEP_BASELIB" = "xyes"
 then
   STEP="baselib"
-  STEPADD=" "
+  STEPADD="  "
 
   if ! unpack_component  zlib         $VERSION_ZLIB           ||
      ! unpack_component  libpng       $VERSION_PNG
@@ -899,9 +944,9 @@ then
     exit 1
   fi
 
-  if ! build_zlib                zlib    zlib                     \
-        "--shared"                                                ||
-     ! build_with_cross_compiler libpng  libpng-$VERSION_PNG
+  if ! build_zlib                zlib      zlib                         \
+        "--shared"                                                      ||
+     ! build_with_cross_compiler libpng    libpng-$VERSION_PNG
   then
     crosser_error "Baselib build failed"
     exit 1
@@ -911,10 +956,55 @@ then
   then
     log_write 1 "Some parts of baselib step not available for $LIBC_MODE based builds, skipping"
   else
-    if ! unpack_component libxml2 $VERSION_LIBXML2                  ||
-       ! build_with_cross_compiler libxml2 libxml2-$VERSION_LIBXML2
+    STEP="bl(im)"
+    STEPADD="   "
+
+    # If native step already executed, libxslt already unpacked
+    if test "x$STEP_NATIVE" != "xyes"
     then
-      crosser_error "Baselib build failed"
+      if ! unpack_component libgpg-error $VERSION_GPGERROR ||
+         ! unpack_component libgcrypt $VERSION_LIBGCRYPT   ||
+         ! unpack_component libxslt $VERSION_LIBXSLT
+      then
+        crosser_error "Unpacking of baselib source packages failed"
+        exit 1
+      fi
+    fi
+
+    if ! unpack_component libxml2 $VERSION_LIBXML2                        ||
+       ! build_with_cross_compiler libxml2 libxml2-$VERSION_LIBXML2       \
+         "--prefix=$CROSSER_IM_PFX" "" "/"
+       ! build_with_cross_compiler libgpg-error \
+         libgpg-error-$VERSION_GPGERROR \
+         "--prefix=$CROSSER_IM_PFX" "" "/"                                ||
+       ! build_with_cross_compiler libgcrypt libgcrypt-$VERSION_LIBGCRYPT \
+         "--prefix=$CROSSER_IM_PFX --with-gpg-error-prefix=$CROSSER_IM_PFX --disable-asm" \
+         "" "/" ||
+       ! link_im_host_prog libgcrypt-config ||
+       ! build_with_cross_compiler libxslt   libxslt-$VERSION_LIBXSLT     \
+         "--prefix=$CROSSER_IM_PFX --with-libxml-prefix=$CROSSER_IM_PFX"  \
+         "" "/" ||
+       ! unpack_component jpeg $VERSION_JPEG "" "jpegsrc.v${VERSION_JPEG}" ||
+       ! build_with_cross_compiler jpeg      jpeg-$VERSION_JPEG           \
+         "--prefix=$CROSSER_IM_PFX" "" "/"
+    then
+      crosser_error "Baselib intermediate part build failed"
+      exit 1
+    fi
+
+    STEP="bl(tgt)"
+    STEPADD="  "
+
+    if ! build_with_cross_compiler libxml2 libxml2-$VERSION_LIBXML2       ||
+       ! build_with_cross_compiler libgpg-error \
+         libgpg-error-$VERSION_GPGERROR                                   ||
+       ! build_with_cross_compiler libgcrypt libgcrypt-$VERSION_LIBGCRYPT \
+         "--with-gpg-error-prefix=$CROSSER_IM_PFX --disable-asm"          ||
+       ! build_with_cross_compiler libxslt   libxslt-$VERSION_LIBXSLT     \
+         "--with-libxml-prefix=$CROSSER_IM_PFX"                           ||
+       ! build_with_cross_compiler jpeg      jpeg-$VERSION_JPEG
+    then
+      crosser_error "Baselib target build failed"
       exit 1
     fi
   fi
@@ -923,7 +1013,7 @@ fi
 if test "x$STEP_XORG" = "xyes"
 then
   STEP="xorg"
-  STEPADD="    "
+  STEPADD="     "
 
   if test "x$E_GLIBC" != "xyes"
   then
@@ -934,7 +1024,31 @@ then
        ! unpack_component xextproto $VERSION_XORG_XEXTPROTO                    ||
        ! build_with_cross_compiler xextproto xextproto-$VERSION_XORG_XEXTPROTO ||
        ! unpack_component xtrans $VERSION_XORG_XTRANS                          ||
-       ! build_with_cross_compiler xtrans xtrans-$VERSION_XORG_XTRANS
+       ! build_with_cross_compiler xtrans xtrans-$VERSION_XORG_XTRANS          ||
+       ! unpack_component libpthread-stubs $VERSION_PTHREAD_STUBS              ||
+       ! build_with_cross_compiler libpthread-stubs libpthread-stubs-$VERSION_PTHREAD_STUBS ||
+       ! unpack_component libXau    $VERSION_XORG_LIBXAU                            ||
+       ! build_with_cross_compiler libXau libXau-$VERSION_XORG_LIBXAU
+    then
+      crosser_error "Xorg build failed"
+      exit 1
+    fi
+
+    STEP="xorg(im)"
+    STEPADD=" "
+
+    if ! unpack_component xcb-proto $VERSION_XCB_PROTO                         ||
+       ! build_with_cross_compiler xcb-proto xcb-proto-$VERSION_XCB_PROTO      \
+         "--prefix=$CROSSER_IM_PFX" "" "/"
+    then
+      crosser_error "Xorg intermediate part build failed"
+      exit 1
+    fi
+
+    STEP="xorg(tgt)"
+    STEPADD=""
+
+    if ! build_with_cross_compiler xcb-proto xcb-proto-$VERSION_XCB_PROTO
     then
       crosser_error "Xorg build failed"
       exit 1
@@ -945,7 +1059,7 @@ fi
 if test "x$STEP_GTK" = "xyes"
 then
   STEP="gtk"
-  STEPADD="     "
+  STEPADD="      "
 
   if test "x$E_GLIBC" != "xyes"
   then
@@ -954,51 +1068,64 @@ then
     GLIB_VARS="$(read_configure_vars glib)"
     log_write 4 "Glib variables: $GLIB_VARS"
 
-    if ! unpack_component          glib          $VERSION_GLIB                           ||
-       ! ( ! cmp_versions $VERSION_GLIB 2.18.0    ||
-           ( patch_src glib-$VERSION_GLIB glib_gmoddef &&
-             autogen_component glib    $VERSION_GLIB "aclocal automake autoconf" ))      ||
-       ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
-         "--prefix=/usr $GLIB_VARS"
+    # If native step already executed, glib already unpacked
+    if test "x$STEP_NATIVE" != "xyes"
     then
-      crosser_error "Glib build failed"
-      exit 1
+      if ! unpack_component          glib          $VERSION_GLIB
+      then
+        crosser_error "Glib unpacking failed"
+        exit 1
+      fi
     fi
 
-    STEP="gtk(ft1)"
-    STEPADD=""
-    if ! unpack_component          freetype      $VERSION_FREETYPE                       ||
+    STEP="gtk(im)"
+    STEPADD="  "
+    if ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
+         "--prefix=$CROSSER_IM_PFX $GLIB_VARS" "" "/"                                    ||
+       ! unpack_component          freetype      $VERSION_FREETYPE                       ||
        ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              \
-         "--prefix=$PREFIX/interm" "" "/"
+         "--prefix=$CROSSER_IM_PFX" "" "/" ||
+       ! unpack_component          pixman        $VERSION_PIXMAN                         ||
+       ! build_with_cross_compiler pixman        pixman-$VERSION_PIXMAN                  \
+         "--prefix=$CROSSER_IM_PFX --disable-gtk" "" "/" ||
+       ! unpack_component          expat         $VERSION_EXPAT                          ||
+       ! build_with_cross_compiler expat         expat-$VERSION_EXPAT                    \
+         "--prefix=$CROSSER_IM_PFX" "" "/" ||
+       ! unpack_component          fontconfig    $VERSION_FONTCONFIG                     ||
+       ! patch_src fontconfig-$VERSION_FONTCONFIG fontconfig_cross                       ||
+       ! build_with_cross_compiler fontconfig                                            \
+          fontconfig-$VERSION_FONTCONFIG                                                 \
+          "--prefix=$CROSSER_IM_PFX --with-freetype-config=$CROSSER_IM_PFX/bin/freetype-config --with-arch=$TARGET" \
+          "install" "/"
     then
-      crosser_error "Intermediate freetype build failed"
+      crosser_error "Intermediate part of gtk+ chain build failed"
       exit 1
     fi
 
-    STEP="gtk(ft2)"
-    STEPADD=""
-    if ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE
+    STEP="gtk(tgt)"
+    STEPADD=" "
+    if ! build_with_cross_compiler glib          glib-$VERSION_GLIB                      \
+         "--prefix=/usr $GLIB_VARS" ||
+       ! build_with_cross_compiler freetype      freetype-$VERSION_FREETYPE              ||
+       ! build_with_cross_compiler pixman        pixman-$VERSION_PIXMAN                  \
+         "--prefix=/usr --disable-gtk" ||
+       ! build_with_cross_compiler expat         expat-$VERSION_EXPAT                    ||
+       ! build_with_cross_compiler fontconfig                                            \
+          fontconfig-$VERSION_FONTCONFIG                                                 \
+          "--prefix=/usr --with-freetype-config=$CROSSER_IM_PFX/bin/freetype-config --with-arch=$TARGET" \
+          "install"
     then
-      crosser_error "Target freetype build failed"
+      crosser_error "Target gtk+ chain build failed"
       exit 1
     fi
 
     STEP="gtk"
     STEPADD="     "
-    if ! unpack_component          expat         $VERSION_EXPAT                          ||
-       ! build_with_cross_compiler expat         expat-$VERSION_EXPAT                    ||
-       ! unpack_component          pixman        $VERSION_PIXMAN                         ||
-       ! build_with_cross_compiler pixman        pixman-$VERSION_PIXMAN                  \
-         "--prefix=/usr --disable-gtk"                                                   ||
-       ! unpack_component          fontconfig    $VERSION_FONTCONFIG                     ||
-       ! patch_src fontconfig-$VERSION_FONTCONFIG fontconfig_cross                       ||
-       ! patch_src fontconfig-$VERSION_FONTCONFIG fontconfig_libtool                     ||
-       ! autogen_component fontconfig $VERSION_FONTCONFIG                                \
-         "libtoolize aclocal automake autoconf"                                          ||
-       ! build_with_cross_compiler fontconfig                                            \
-          fontconfig-$VERSION_FONTCONFIG                                                 \
-          "--prefix=/usr --with-freetype-config=$PREFIX/interm/bin/freetype-config --with-arch=$TARGET" \
-          "install"
+    if ! unpack_component          atk           $VERSION_ATK                            ||
+       ! build_with_cross_compiler atk           atk-$VERSION_ATK                        \
+         "--disable-glibtest" ||
+       ! unpack_component          tiff          $VERSION_TIFF                           ||
+       ! build_with_cross_compiler tiff          tiff-$VERSION_TIFF
     then
       crosser_error "gtk+ chain build failed"
       exit 1
@@ -1024,15 +1151,39 @@ then
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_cfg                   ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_gentoo_k26            ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_gentoo_k2628          ||
+       ! patch_src svgalib-$VERSION_SVGALIB svgalib_gentoo_k2636          ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_arm_outsb             ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_nostrip               ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_glinc                 ||
        ! patch_src svgalib-$VERSION_SVGALIB svgalib_round                 ||
        ! build_svgalib             svgalib svgalib-$VERSION_SVGALIB       \
           "clean install"                                                 ||
-       ! unpack_component          SDL       $VERSION_SDL                 ||
-       ! build_with_cross_compiler SDL       SDL-$VERSION_SDL             ||
-       ! unpack_component          SDL_image $VERSION_SDL_IMAGE           ||
+       ! unpack_component          SDL       $VERSION_SDL
+    then
+      crosser_error "sdl stack build failed"
+      exit 1
+    fi
+
+    STEP="sdl(im)"
+    STEPADD=" "
+    if ! build_with_cross_compiler SDL       SDL-$VERSION_SDL             \
+         "--prefix=$CROSSER_IM_PFX" "" "/"
+    then
+      crosser_error "Intermediate SDL build failed"
+      exit 1
+    fi
+
+    STEP="sdl(tgt)"
+    STEPADD=""
+    if ! build_with_cross_compiler SDL       SDL-$VERSION_SDL
+    then
+      crosser_error "Target SDL build failed"
+      exit 1
+    fi
+
+    STEP="sdl"
+    STEPADD="     "
+    if ! unpack_component          SDL_image $VERSION_SDL_IMAGE           ||
        ! build_with_cross_compiler SDL_image SDL_image-$VERSION_SDL_IMAGE
     then
       crosser_error "sdl stack build failed"
