@@ -2,12 +2,12 @@
 
 # dllstack.sh: Cross-compile set of libraries for Windows target.
 #
-# (c) 2008-2010 Marko Lindqvist
+# (c) 2008-2011 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 #
 
-CROSSER_MAINDIR=$(cd $(dirname $0) ; pwd)
+CROSSER_MAINDIR="$(cd "$(dirname "$0")" ; pwd)"
 
 if ! test -e "$CROSSER_MAINDIR/CrosserVersion"
 then
@@ -18,32 +18,32 @@ export CROSSER_OPTION_JPEG=on
 
 if test "x$1" = "x-h" || test "x$1" = "x--help"
 then
-  echo "Usage: $(basename $0) [[-h|--help]|[-v|--version]|[install prefix]] [versionset]"
+  echo "Usage: $(basename "$0") [[-h|--help]|[-v|--version]|[install prefix]] [versionset]"
   exit 0
 fi
 
 # In order to give local setup opportunity to override versions,
-# we have to load versionset before build_setup.conf
-# helpers.sh requires environment to be set up by build_setup.conf.
+# we have to load versionset before setup_reader.sh
+# helpers.sh requires environment to be set up by setup_reader.sh.
 if test "x$2" != "x" ; then
   VERSIONSET="$2"
 else
   VERSIONSET="current"
 fi
-if test -e $CROSSER_MAINDIR/setups/$VERSIONSET.versions
+if test -e "$CROSSER_MAINDIR/setups/$VERSIONSET.versions"
 then
-  . $CROSSER_MAINDIR/setups/$VERSIONSET.versions
+  . "$CROSSER_MAINDIR/setups/$VERSIONSET.versions"
 else
-  # Versions being unset do not prevent loading of build_setup.conf and helper.sh,
+  # Versions being unset do not prevent loading of setup_reader.sh and helper.sh,
   # resulting environment would just be unusable for building.
   # We are not going to build anything, but just issuing error message - and for
   # that we read log_error from helpers.sh
   ERR_MSG="Cannot find versionset \"$VERSIONSET.versions\""
 fi
 
-. $CROSSER_MAINDIR/build_setup.conf
-. $CROSSER_MAINDIR/scripts/helpers.sh
-. $CROSSER_MAINDIR/scripts/packethandlers.sh
+. "$CROSSER_MAINDIR/scripts/setup_reader.sh"
+. "$CROSSER_MAINDIR/scripts/helpers.sh"
+. "$CROSSER_MAINDIR/scripts/packethandlers.sh"
 
 # This must be after reading helpers.sh so that $CROSSER_VERSION is set
 if test "x$1" = "x-v" || test "x$1" = "x--version"
@@ -83,6 +83,10 @@ build_component()
 build_component_host()
 {
   build_component_full "host-$1" "$1" "$2" "$3" "" "native"
+
+  # Reset command hash in case it already contained old version of the
+  # just built tool
+  hash -r 
 }
 
 # $1 - Build dir
@@ -103,14 +107,14 @@ build_component_full()
     return 1
   fi
 
-  BUILDDIR="$MAINBUILDDIR/$1"
+  BUILDDIR="$CROSSER_BUILDDIR/$1"
   if ! mkdir -p "$BUILDDIR"
   then
     log_error "Failed to create directory $BUILDDIR"
     return 1
   fi
-  cd $BUILDDIR
-  SRCDIR="$MAINSRCDIR/$SUBDIR"
+  cd "$BUILDDIR"
+  SRCDIR="$CROSSER_SRCDIR/$SUBDIR"
 
   if test "x$6" != "xnative"
   then
@@ -123,19 +127,22 @@ build_component_full()
     unset LDFLAGS
   fi
 
-  log_write 1 "Configuring $1"
-  log_write 3 "  Options: \"$CONFOPTIONS\""
-  log_flags
-
-  if ! $SRCDIR/configure $CONFOPTIONS >>$LOGDIR/stdout.log 2>>$LOGDIR/stderr.log
+  if test -x "$SRCDIR/configure"
   then
-    log_error "Configure for $1 failed"
-    return 1
+    log_write 1 "Configuring $1"
+    log_write 3 "  Options: \"$CONFOPTIONS\""
+    log_flags
+
+    if ! "$SRCDIR/configure" $CONFOPTIONS >>$CROSSER_LOGDIR/stdout.log 2>>$CROSSER_LOGDIR/stderr.log
+    then
+      log_error "Configure for $1 failed"
+      return 1
+    fi
   fi
 
   if test "x$5" = "xoverwrite" ; then
     log_write 2 "Copying working libtool to $1"
-    if ! cp $MAINBUILDDIR/libtool/libtool .
+    if ! cp "$CROSSER_BUILDDIR/libtool/libtool" .
     then
       log_error "Failed to copy libtool"
       return 1
@@ -148,19 +155,25 @@ build_component_full()
   log_write 1 "Building $1"
   log_write 3 "  Make targets: [default] install"
 
-  if ! make  >>$LOGDIR/stdout.log 2>>$LOGDIR/stderr.log
+  if ! make  >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log"
   then
     log_error "Make for $1 failed"
     return 1
   fi
 
-  if ! make install >>$LOGDIR/stdout.log 2>>$LOGDIR/stderr.log
+  if ! make install >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log"
   then
     log_error "Install for $1 failed"
     return 1
   fi
 }
 
+# Build zlib
+#
+# $1 - Package name
+# $2 - Version
+# $3 - Configure options
+#
 build_zlib()
 {
   log_packet "$1"
@@ -177,9 +190,9 @@ build_zlib()
   export RANLIB=$TARGET-ranlib
   export AR=$TARGET-ar
 
-  if ! cd $MAINSRCDIR/$SUBDIR
+  if ! cd "$CROSSER_SRCDIR/$SUBDIR"
   then
-    log_error "Cannot change to directory $MAINSRCDIR/$SUBDIR"
+    log_error "Cannot change to directory $CROSSER_SRCDIR/$SUBDIR"
     return 1
   fi
 
@@ -195,7 +208,7 @@ build_zlib()
   log_write 3 "  Options: \"$CONFOPTIONS\""
   log_flags
 
-  if ! ./configure $CONFOPTIONS >>$LOGDIR/stdout.log 2>>$LOGDIR/stderr.log
+  if ! ./configure $CONFOPTIONS >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log"
   then
     log_error "Configure for $1 failed"
     return 1
@@ -204,26 +217,74 @@ build_zlib()
   log_write 1 "Building $1"
   log_write 3 "  Make targets: [default] install"
 
-  if ! make >>$LOGDIR/stdout.log 2>>$LOGDIR/stderr.log
+  if ! make >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log"
   then
     log_error "Make for $1 failed"
     return 1
   fi
 
-  if ! make install  >>$LOGDIR/stdout.log 2>>$LOGDIR/stderr.log
+  if ! make install >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log"
   then
     log_error "Install for $1 failed"
     return 1
   fi
 
-  if ! cp $DLLSPREFIX/lib/libz.dll* $DLLSPREFIX/bin/ ||
-     ! mv $DLLSPREFIX/lib/libz.a    $DLLSPREFIX/bin/
+  if ! cp "$DLLSPREFIX/lib/libz.dll"* "$DLLSPREFIX/bin/" ||
+     ! mv "$DLLSPREFIX/lib/libz.a"    "$DLLSPREFIX/bin/"
   then
     log_error "Failed to move libz dll:s to correct directory"
     return 1
   fi
 }
 
+# Build bzip2
+#
+# $1 - Package name
+# $2 - Version
+#
+build_bzip2()
+{
+  log_packet "$1"
+
+  SUBDIR="$(src_subdir $1 $2)"
+
+  if test "x$SUBDIR" = "x"
+  then
+    log_error "Cannot find srcdir for $1 version $2"
+    return 1
+  fi
+
+  if ! cd "$CROSSER_SRCDIR/$SUBDIR"
+  then
+    log_error "Cannot change to directory $CROSSER_SRCDIR/$SUBDIR"
+    return 1
+  fi
+
+  export CC=$TARGET-gcc
+  export RANLIB=$TARGET-ranlib
+  export AR=$TARGET-ar
+  export PREFIX=$DLLSPREFIX
+  export CPPFLAGS="-isystem $DLLSPREFIX/include -isystem $TGT_HEADERS $TGT_MARCH $USER_CPPFLAGS"
+  export LDFLAGS="-L$DLLSPREFIX/lib $USER_LDFLAGS"
+
+  log_write 1 "Building $1"
+  log_write 3 "  Make targets: libbz2.a bzip2 bzip2recover & install"
+  log_flags
+
+  if ! make libbz2.a bzip2 bzip2recover \
+       >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log"
+  then
+    log_error "Make for $1 failed"
+    return 1
+  fi
+
+  if ! make install >> "$CROSSER_LOGDIR/stdout.log" 2>> "$CROSSER_LOGDIR/stderr.log"
+  then
+    log_error "Install for $1 failed"
+    return 1
+  fi
+}
+
 # Update one autotools auxiliary file for component
 #
 # $1 - Source directory in source hierarchy
@@ -231,9 +292,9 @@ build_zlib()
 update_aux_file()
 {
   # Update only those files that already exist in target directory
-  if test -e $MAINSRCDIR/$1/$2 ; then
+  if test -e "$CROSSER_SRCDIR/$1/$2" ; then
     log_write 2 "Updating $2"
-    if ! cp $CROSSER_MAINDIR/scripts/aux/$2 $MAINSRCDIR/$1/
+    if ! cp "$CROSSER_MAINDIR/scripts/aux/$2" "$CROSSER_SRCDIR/$1/"
     then
       return 1
     fi
@@ -256,10 +317,10 @@ update_aux_files() {
     return 1
   fi
 
-  if ! update_aux_file $SUBDIR config.guess ||
-     ! update_aux_file $SUBDIR config.sub   ||
-     ! update_aux_file $SUBDIR install-sh   ||
-     ! update_aux_file $SUBDIR ltmain.sh
+  if ! update_aux_file "$SUBDIR" config.guess ||
+     ! update_aux_file "$SUBDIR" config.sub   ||
+     ! update_aux_file "$SUBDIR" install-sh   ||
+     ! update_aux_file "$SUBDIR" ltmain.sh
   then
     log_error "Failed to update auxiliary files in directory $SUBDIR"
     return 1
@@ -268,7 +329,7 @@ update_aux_files() {
 
 cd $(dirname $0)
 
-if ! . $CROSSER_MAINDIR/setups/native.sh ; then
+if ! . "$CROSSER_MAINDIR/setups/native.sh" ; then
   log_error "Failed to read $CROSSER_MAINDIR/setups/native.sh"
   exit 1
 fi
@@ -297,9 +358,9 @@ else
   TARGET="$TARGET_ARCH-$TARGET_VENDOR-$TARGET_OS"
 fi
 
-if test -d /usr/$TARGET/include
+if test -d "/usr/$TARGET/include"
 then
-  export TGT_HEADERS=/usr/$TARGET/include
+  export TGT_HEADERS="/usr/$TARGET/include"
 fi
 
 TGT_MARCH="-march=$TARGET_ARCH"
@@ -311,8 +372,8 @@ if test "x$DLLSHOST_PREFIX" = "x" && test "x$LSHOST_PREFIX" != "x" ; then
   DLLSHOST_PREFIX="$LSHOST_PREFIX"
 fi
 
-export DLLSPREFIX=$(setup_prefix_default "$HOME/.crosser/<VERSION>/winstack" "$DLLSPREFIX")
-export NATIVE_PREFIX=$(setup_prefix_default "$HOME/.crosser/$CROSSER_VERSION/dllshost" \
+export DLLSPREFIX=$(setup_prefix_default "$HOME/.crosser/<VERSION>/<VERSIONSET>/winstack" "$DLLSPREFIX")
+export NATIVE_PREFIX=$(setup_prefix_default "$HOME/.crosser/<VERSION>/<VERSIONSET>/dllshost" \
                        "$DLLSHOST_PREFIX")
 
 export USER_CPPFLGS="$CPPFLAGS"
@@ -321,45 +382,45 @@ export USER_CFLAGS="$CFLAGS"
 export USER_CXXFLAGS="$CXXFLAGS"
 
 log_write 2 "Install:    \"$DLLSPREFIX\""
-log_write 2 "Src:        \"$MAINSRCDIR\""
-log_write 2 "Log:        \"$LOGDIR\""
-log_write 2 "Build:      \"$MAINBUILDDIR\""
+log_write 2 "Src:        \"$CROSSER_SRCDIR\""
+log_write 2 "Log:        \"$CROSSER_LOGDIR\""
+log_write 2 "Build:      \"$CROSSER_BUILDDIR\""
 log_write 2 "Versionset: \"$VERSIONSET\""
 
-if ! remove_dir "$MAINSRCDIR"    ||
-   ! remove_dir "$MAINBUILDDIR"  ||
-   ! remove_dir "$DLLSPREFIX"    ||
+if ! remove_dir "$CROSSER_SRCDIR"    ||
+   ! remove_dir "$CROSSER_BUILDDIR"  ||
+   ! remove_dir "$DLLSPREFIX"        ||
    ! remove_dir "$NATIVE_PREFIX"
 then
   log_error "Failed to remove old directories"
   exit 1
 fi
 
-if ! mkdir -p $MAINSRCDIR
+if ! mkdir -p "$CROSSER_SRCDIR"
 then
-  log_error "Cannot create directory $MAINSRCDIR"
+  log_error "Cannot create directory $CROSSER_SRCDIR"
   exit 1
 fi
 
-if ! mkdir -p $MAINBUILDDIR
+if ! mkdir -p "$CROSSER_BUILDDIR"
 then
-  log_error "Cannot create directory $MAINBUILDDIR"
+  log_error "Cannot create directory $CROSSER_BUILDDIR"
   exit 1
 fi
 
-if ! mkdir -p $DLLSPREFIX/man/man1
+if ! mkdir -p "$DLLSPREFIX/man/man1"
 then
   log_error "Cannot create target directory hierarchy under $DLLSPREFIX"
   exit 1
 fi
 
-if ! mkdir -p $NATIVE_PREFIX/bin
+if ! mkdir -p "$NATIVE_PREFIX/bin"
 then
   log_error "Cannot create host directory hierarchy under $NATIVE_PREFIX"
   exit 1
 fi
 
-export PATH=$NATIVE_PREFIX/bin:$PATH
+export PATH="$NATIVE_PREFIX/bin:$PATH"
 
 if ! packetdir_check
 then
@@ -369,20 +430,29 @@ fi
 
 if test "x$CROSSER_DOWNLOAD" = "xyes"
 then
-  if ! (cd $PACKETDIR && $CROSSER_MAINDIR/scripts/download_packets.sh "win" )
+  if ! (cd "$PACKETDIR" && "$CROSSER_MAINDIR/scripts/download_packets.sh" "win" )
   then
     log_error "Downloading packets failed"
     exit 1
   fi
 fi
 
-export PKG_CONFIG_LIBDIR=$DLLSPREFIX/lib/pkgconfig
+export PKG_CONFIG_LIBDIR="$DLLSPREFIX/lib/pkgconfig"
 
 BASEVER_LIBTOOL="$(basever_libtool $VERSION_LIBTOOL)"
+GLIB_VARS="$(read_configure_vars glib)"
 
 # glib_acsizeof -patch is required only when running autogen for glib
 if ! unpack_component     autoconf   $VERSION_AUTOCONF      ||
    ! build_component_host autoconf   $VERSION_AUTOCONF      ||
+   ! unpack_component     automake   $VERSION_AUTOMAKE      ||
+   ! build_component_host automake   $VERSION_AUTOMAKE      ||
+   ! unpack_component     libtool    $VERSION_LIBTOOL       ||
+   ! build_component_host libtool    $BASEVER_LIBTOOL       ||
+   ! unpack_component     pkg-config $VERSION_PKG_CONFIG    ||
+   ! (! cmp_versions $VERSION_PKG_CONFIG 0.25 ||
+      patch_src pkg-config-$VERSION_PKG_CONFIG pkgconfig_ac266) ||
+   ! build_component_host pkg-config $VERSION_PKG_CONFIG    ||
    ! unpack_component     glib       $VERSION_GLIB          ||
    ! (! cmp_versions $VERSION_GLIB 2.18.0 ||
         ( patch_src glib-$VERSION_GLIB glib_gmoddef  &&
@@ -405,6 +475,12 @@ if ! unpack_component  libtool    $VERSION_LIBTOOL                   ||
    ! patch_src zlib               zlib_nolibc                        ||
    ! patch_src zlib               zlib_dllext                        ||
    ! build_zlib        zlib                                          ||
+   ! unpack_component  bzip2      $VERSION_BZIP2                     ||
+   ! patch_src bzip2-$VERSION_BZIP2 bzip2_unhardcodecc               ||
+   ! patch_src bzip2-$VERSION_BZIP2 bzip2_incpathsep                 ||
+   ! build_bzip2       bzip2      $VERSION_BZIP2                     ||
+   ! unpack_component  curl       $VERSION_CURL                      ||
+   ! build_component   curl       $VERSION_CURL                      ||
    ! unpack_component  libpng     $VERSION_PNG                       ||
    ! patch_src libpng-$VERSION_PNG png_symbol_prefix                 ||
    ! autogen_component libpng     $VERSION_PNG                       ||
@@ -415,7 +491,7 @@ if ! unpack_component  libtool    $VERSION_LIBTOOL                   ||
          patch_src gettext-$VERSION_GETTEXT gettext_no_rpl_optarg )) ||
    ! (export LIBS="-liconv" && build_component gettext  $VERSION_GETTEXT) ||
    ! build_component   glib       $VERSION_GLIB             \
-       "glib_cv_stack_grows=yes"
+       "$GLIB_VARS"
 then
   log_error "Build failed"
   exit 1
@@ -423,24 +499,12 @@ fi
 
 if test "x$CROSSER_OPTION_JPEG" = "xon"
 then
-  JPEG_BASENAME=libjpeg$VERSION_JPEG
-  if ! unpack_component     $JPEG_BASENAME $VERSION_JPEG_DEB
+  if ! unpack_component jpeg $VERSION_JPEG "" "jpegsrc.v${VERSION_JPEG}"
   then
     log_error "Libjpeg download failed"
     exit 1
   fi
-  if is_smaller_version $VERSION_JPEG 7
-  then
-    if ! patch_src            $JPEG_BASENAME jpeg_ltcompile           ||
-       ! patch_src            $JPEG_BASENAME jpeg_ar                  ||
-       ! patch_src            $JPEG_BASENAME jpeg_noundef             ||
-       ! update_aux_files     $JPEG_BASENAME $VERSION_JPEG
-    then
-      log_error "Libjpeg preparation failed"
-      exit 1
-    fi
-  fi
-  if ! build_component_full $JPEG_BASENAME $JPEG_BASENAME  $VERSION_JPEG "--enable-shared" "overwrite"
+  if ! build_component jpeg $VERSION_JPEG "--enable-shared"
   then
     log_error "Libjpeg build failed"
     exit 1
@@ -461,9 +525,7 @@ then
   exit 1
 fi
 
-if ! patch_src tiff-$VERSION_TIFF tiff_config_headers       ||
-   ! ( ! cmp_versions $VERSION_TIFF 4.0.0alpha ||
-       patch_src tiff-$VERSION_TIFF tiff4alpha_largefile )
+if ! patch_src tiff-$VERSION_TIFF tiff_config_headers
 then
   log_error "Tiff patching failed"
   exit 1
@@ -472,7 +534,7 @@ fi
 if is_smaller_version $VERSION_TIFF 3.9.0
 then
   log_write 1 "Removing upstream libtiff config"
-  if ! rm $MAINSRCDIR/tiff-$VERSION_TIFF/libtiff/tiffconf.h
+  if ! rm "$CROSSER_SRCDIR/tiff-$VERSION_TIFF/libtiff/tiffconf.h"
   then
     log_error "Failed to remove old tiffconf.h"
     exit 1
@@ -513,7 +575,9 @@ if ! unpack_component  fontconfig $VERSION_FONTCONFIG               ||
    ! build_component   pixman     $VERSION_PIXMAN                 \
      "--disable-gtk"                                              ||
    ! unpack_component  cairo      $VERSION_CAIRO                  ||
-   ! rm -f $MAINSRCDIR/cairo-$VERSION_CAIRO/src/cairo-features.h  ||
+   ! rm -f "$CROSSER_SRCDIR/cairo-$VERSION_CAIRO/src/cairo-features.h" ||
+   ! ( is_smaller_version $VERSION_CAIRO 1.10.0 ||
+       patch_src         cairo-$VERSION_CAIRO cairo_ffs )         ||
    ! build_component   cairo      $VERSION_CAIRO                  \
      "--disable-xlib --enable-win32"                              ||
    ! unpack_component  pango      $VERSION_PANGO                  ||
@@ -529,7 +593,10 @@ then
   exit 1
 fi
 
-if ! unpack_component  gtk+       $VERSION_GTK                    ||
+if ! ( is_smaller_version $VERSION_GTK 2.22.0 ||
+       ( unpack_component gdk-pixbuf $VERSION_GDK_PIXBUF &&
+         build_component gdk-pixbuf $VERSION_GDK_PIXBUF ))        ||
+   ! unpack_component  gtk+       $VERSION_GTK                    ||
    ! ( is_minimum_version $VERSION_GTK      2.12.10 ||
        patch_src gtk+-$VERSION_GTK          gtk_blddir )          ||
    ! ( is_minimum_version $VERSION_GTK      2.13.2 ||
@@ -564,7 +631,7 @@ log_write 1 "Creating setup.bat"
   echo -n -e "if not exist etc\pango mkdir etc\pango\r\n"
   echo -n -e "bin\pango-querymodules.exe > etc\pango\pango.modules\r\n"
   echo -n -e "bin\gdk-pixbuf-query-loaders.exe > etc\gtk-2.0\gdk-pixbuf.loaders\r\n"
-) > $DLLSPREFIX/setup.bat
+) > "$DLLSPREFIX/setup.bat"
 log_write 1 "IMPORTANT: Remember to create configuration files when installing to target"
 
 if ! unpack_component  SDL        $VERSION_SDL          ||
