@@ -2,7 +2,7 @@
 
 # download_packets.sh: Source package downloader
 #
-# (c) 2008-2010 Marko Lindqvist
+# (c) 2008-2011 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 
@@ -30,6 +30,7 @@ download_file() {
     then
       sed "s/: .* : $2\$/: $TIMEPART : $2/" filelist.txt > filelist.tmp
       mv filelist.tmp filelist.txt
+      APPEND=no
     else
       APPEND=yes
     fi
@@ -42,13 +43,13 @@ download_file() {
     echo "$TIMEPART : $TIMEPART : $2" >> filelist.txt
   fi
 
-  if test -f $DLDIR/$2
+  if test -f "$DLDIR/$2"
   then
     echo "Already has $2, skipping"
     return 0
   fi
 
-  if ! ( cd $DLDIR && wget -T 180 -t 2 "$1$2" ) ; then
+  if ! ( cd "$DLDIR" && wget -T 180 -t 2 "$1$2" ) ; then
     echo "Download of $2 failed" >&2
     return 1
   fi
@@ -72,7 +73,7 @@ download_packet() {
     fi
 
     FILELIST_SECTION=no
-    cat $DLDIR/$DLFILENAME |
+    cat "$DLDIR/$DLFILENAME" |
     ( while read PART1 PART2 PART3
       do
         if test "x$FILELIST_SECTION" = "xyes"
@@ -93,10 +94,6 @@ download_packet() {
         fi
       done
     )
-
-    DLFILE2="${2}_$3.diff.gz"
-    BASEVERSION=$(echo $3 | sed 's/-.*//')
-    DLFILE3="${2}_$BASEVERSION.orig.tar.gz"
   else
     if test "x$4" = "x" ; then
       DLFILENAME="$3"
@@ -166,11 +163,7 @@ download_needed() {
   fi
   for STEP in $STEPLIST
   do
-    if "x$2" = "xlibjpeg$VERSION_JPEG" ; then
-      BASENAME=libjpeg
-    else
-      BASENAME=$2
-    fi
+    BASENAME=$2
     if belongs_to_step $BASENAME $STEP ; then
       download_packet "$1" "$2" "$PACKVER" "$4"
       return $?
@@ -221,14 +214,14 @@ download_patches() {
   return 2
 }
 
-CROSSER_MAINDIR="$(cd $(dirname $0)/.. ; pwd)"
+CROSSER_MAINDIR="$(cd "$(dirname "$0")/.." ; pwd)"
 
 if ! test -e "$CROSSER_MAINDIR/CrosserVersion"
 then
   CROSSER_MAINDIR="/usr/share/crosser"
 fi
 
-if ! . $CROSSER_MAINDIR/scripts/helpers.sh
+if ! . "$CROSSER_MAINDIR/scripts/helpers.sh"
 then
   echo "Failed to read $CROSSER_MAINDIR/scripts/helpers.sh" >&2
   exit 1
@@ -241,16 +234,17 @@ elif test "x$1" = "x" ; then
 fi
 
 if test "x$HELP_RETURN" != "x" ; then
-  echo "Usage: $(basename $0) <step> [versionset]"
-  echo "       $(basename $0) --packet <name> [version] [patches]"
+  echo "Usage: $(basename "$0") <step> [versionset]"
+  echo "       $(basename "$0") --packet <name> [version] [patches]"
   echo
   echo " Possible steps:"
   echo "  - native"
   echo "  - chain"
   echo "  - baselib"
+  echo "  - xorg"
   echo "  - gtk"
-  echo "  - win"
   echo "  - sdl"
+  echo "  - win"
 
   exit $HELP_RETURN
 fi
@@ -288,13 +282,13 @@ fi
 
 if test "x$VERSIONSET" != "x"
 then
-  if ! test -e $CROSSER_MAINDIR/setups/$VERSIONSET.versions
+  if ! test -e "$CROSSER_MAINDIR/setups/$VERSIONSET.versions"
   then
     echo "Versionset $VERSIONSET.versions not found" >&2
     exit 1
   fi
 
-  if ! . $CROSSER_MAINDIR/setups/$VERSIONSET.versions ; then
+  if ! . "$CROSSER_MAINDIR/setups/$VERSIONSET.versions" ; then
     echo "Failed to read list of package versions ($VERSIONSET.versions)" >&2
     exit 1
   fi
@@ -313,7 +307,7 @@ if test "x$MIRRORCONF" != "x" ; then
   fi
 fi
 
-if ! . $CROSSER_MAINDIR/steps/stepfuncs.sh ; then
+if ! . "$CROSSER_MAINDIR/steps/stepfuncs.sh" ; then
   echo "Problem in reading stepfuncs.sh" >&2
   exit 1
 fi
@@ -369,8 +363,10 @@ MIRROR_SAVANNAH="http://download.savannah.gnu.org"
 if test "x$VERSION_SELECTED" != "x"
 then
   case $DOWNLOAD_PACKET in
+    bzip2)       VERSION_BZIP2=$VERSION_SELECTED ;;
     glib)        VERSION_GLIB=$VERSION_SELECTED ;;
     pango)       VERSION_PANGO=$VERSION_SELECTED ;;
+    gdk-pixbuf)  VERSION_GDK_PIXBUF=$VERSION_SELECTED ;;
     gtk+)        VERSION_GTK=$VERSION_SELECTED ;;
     gtk-engines) VERSION_GTK_ENG=$VERSION_SELECTED ;;
     gtk-doc)     VERSION_GTK_DOC=$VERSION_SELECTED ;;
@@ -383,15 +379,17 @@ then
     libtool)     VERSION_LIBTOOL=$VERSION_SELECTED ;;
     mpfr)        VERSION_MPFR=$VERSION_SELECTED ;;
     Python)      VERSION_PYTHON=$VERSION_SELECTED ;;
-    libjpeg*)    VERSION_JPEG=$(echo $VERSION_SELECTED | sed 's/-.*//') ;;
+    jpeg)        VERSION_JPEG=$VERSION_SELECTED ;;
     xproto)      VERSION_XORG_XPROTO=$VERSION_SELECTED ;;
     xextproto)   VERSION_XORG_XEXTPROTO=$VERSION_SELECTED ;;
     xtrans)      VERSION_XORG_XTRANS=$VERSION_SELECTED ;;
+    libXau)      VERSION_XORG_LIBXAU=$VERSION_SELECTED ;;
     libX11)      VERSION_XORG_LIBX11=$VERSION_SELECTED ;;
   esac
 fi
 GLIB_DIR="$(echo $VERSION_GLIB | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 PANGO_DIR="$(echo $VERSION_PANGO | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
+GDK_PB_DIR="$(echo $VERSION_GDK_PIXBUF | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 GTK_DIR="$(echo $VERSION_GTK | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 GTK_ENG_DIR="$(echo $VERSION_GTK_ENG | sed 's/\./ /g' | (read MAJOR MINOR PATCH ; echo -n $MAJOR.$MINOR ))"
 GTK_DOC_DIR="$(echo $VERSION_GTK_DOC | sed 's/\./ /g' | (read MAJOR MINOR ; echo -n $MAJOR.$MINOR ))"
@@ -402,21 +400,36 @@ READLINE_SHORT="$(echo $VERSION_READLINE | sed 's/\.//g')"
 case "x$VERSION_XORG_XPROTO" in
   x) ;;
   x7.0.13) VERSION_XORG=X11R7.4 ;;
+  x7.0.16) VERSION_XORG=X11R7.5 ;;
+  x7.0.20) VERSION_XORG=X11R7.6 ;;
 esac
 
 case "x$VERSION_XORG_XEXTPROTO" in
   x) ;;
   x7.0.3) VERSION_XORG=X11R7.4 ;;
+  x7.1.1) VERSION_XORG=X11R7.5 ;;
+  x7.1.2) VERSION_XORG=X11R7.6 ;;
 esac
 
 case "x$VERSION_XORG_XTRANS" in
   x) ;;
   x1.2.1) VERSION_XORG=X11R7.4 ;;
+  x1.2.5) VERSION_XORG=X11R7.5 ;;
+  x1.2.6) VERSION_XORG=X11R7.6 ;;
+esac
+
+case "x$VERSION_XORG_LIBXAU" in
+  x) ;;
+  x1.0.4) VERSION_XORG=X11R7.4 ;;
+  x1.0.5) VERSION_XORG=X11R7.5 ;;
+  x1.0.6) VERSION_XORG=X11R7.6 ;;
 esac
 
 case "x$VERSION_XORG_LIBX11" in
   x) ;;
   x1.1.5) VERSION_XORG=X11R7.4 ;;
+  x1.3.2) VERSION_XORG=X11R7.5 ;;
+  x1.4.0) VERSION_XORG=X11R7.6 ;;
 esac
 
 if is_minimum_version $VERSION_AUTOMAKE 1.6.1
@@ -474,6 +487,8 @@ download_needed "http://my.arava.co.il/~matan/svgalib/" "svgalib"    "$VERSION_S
 RET="$RET $?"
 download_needed "$MIRROR_DEB/pool/main/z/zlib/"         "zlib"       "$VERSION_ZLIB"       "dsc"
 RET="$RET $?"
+download_needed "http://www.bzip.org/$VERSION_BZIP2/"   "bzip2"      "$VERSION_BZIP2"      "tar.gz"
+RET="$RET $?"
 download_needed "$MIRROR_GNU/readline/"                 "readline"   "$VERSION_READLINE"   "tar.gz"
 RET="$RET $?"
 download_patches "$MIRROR_GNU/readline/readline-$VERSION_READLINE-patches/" \
@@ -484,7 +499,7 @@ download_needed "$MIRROR_GNU/gettext/"                  "gettext"    "$VERSION_G
 RET="$RET $?"
 download_needed "$MIRROR_GNOME/sources/glib/$GLIB_DIR/" "glib"       "$VERSION_GLIB"       "tar.bz2"
 RET="$RET $?"
-download_needed "$MIRROR_DEB/pool/main/libj/libjpeg$VERSION_JPEG/" "libjpeg$VERSION_JPEG"  "$VERSION_JPEG_DEB"       "dsc"
+download_needed "http://www.ijg.org/files/"             "jpeg"       "jpegsrc.v${VERSION_JPEG}.tar.gz"
 RET="$RET $?"
 download_needed "ftp://ftp.remotesensing.org/pub/libtiff/" "tiff"    "$VERSION_TIFF"       "tar.gz"
 RET="$RET $?"
@@ -494,6 +509,8 @@ download_needed "$MIRROR_SAVANNAH/releases/freetype/"   "freetype"   "$VERSION_F
 RET="$RET $?"
 download_needed "http://fontconfig.org/release/"        "fontconfig" "$VERSION_FONTCONFIG" "tar.gz"
 RET="$RET $?"
+download_needed "http://curl.haxx.se/download/"         "curl"       "$VERSION_CULR" "tar.bz2"
+RET="$RET $?"
 download_needed "http://cairographics.org/releases/"    "pixman"     "$VERSION_PIXMAN"     "tar.gz"
 RET="$RET $?"
 download_needed "http://cairographics.org/releases/"    "cairo"      "$VERSION_CAIRO"      "tar.gz"
@@ -504,6 +521,8 @@ download_needed "$MIRROR_GNOME/sources/atk/$ATK_DIR/"   "atk"        "$VERSION_A
 RET="$RET $?"
 download_needed "$MIRROR_GNOME/sources/gtk-doc/$GTK_DOC_DIR/" "gtk-doc" "$VERSION_GTK_DOC" "tar.bz2"
 RET="$RET $?"
+download_needed "$MIRROR_GNOME/sources/gdk-pixbuf/$GDK_PB_DIR/" "gdk-pixbuf" "$VERSION_GDK_PIXBUF"  "tar.bz2" 
+RET="$RET $?"
 download_needed "$MIRROR_GNOME/sources/gtk+/$GTK_DIR/"  "gtk+"       "$VERSION_GTK"        "tar.bz2"
 RET="$RET $?"
 download_needed "$MIRROR_GNOME/sources/gtk-engines/$GTK_ENG_DIR/"  "gtk-engines" "$VERSION_GTK_ENG" "tar.bz2"
@@ -520,6 +539,10 @@ download_needed "ftp://xmlsoft.org/libxml2/"                 "libxml2"   "$VERSI
 RET="$RET $?"
 download_needed "ftp://xmlsoft.org/libxslt/"                 "libxslt"   "$VERSION_LIBXSLT"        "tar.gz"
 RET="$RET $?"
+download_needed "http://xcb.freedesktop.org/dist/"           "libpthread-stubs" "$VERSION_PTHREAD_STUBS" "tar.bz2"
+RET="$RET $?"
+download_needed "http://xcb.freedesktop.org/dist/"           "xcb-proto" "$VERSION_XCB_PROTO"      "tar.bz2"
+RET="$RET $?"
 download_needed "http://xcb.freedesktop.org/dist/"           "libxcb"    "$VERSION_LIBXCB"         "tar.bz2"
 RET="$RET $?"
 download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "xproto"    "$VERSION_XORG_XPROTO"    "tar.bz2"
@@ -528,8 +551,14 @@ download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "xextproto" "$VERSI
 RET="$RET $?"
 download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "xtrans"    "$VERSION_XORG_XTRANS" "tar.bz2"
 RET="$RET $?"
+download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "libXau"    "$VERSION_XORG_LIBXAU" "tar.bz2"
+RET="$RET $?"
 download_needed "$MIRROR_XORG/$VERSION_XORG/src/everything/" "libX11"    "$VERSION_XORG_LIBX11" "tar.bz2"
 RET="$RET $?"
+download_needed "ftp://ftp.gnupg.org/gcrypt/libgpg-error/"   "libgpg-error" "$VERSION_GPGERROR" "tar.bz2"
+RET="$RET $?"
+download_needed "ftp://ftp.gnupg.org/gcrypt/libgcrypt/"      "libgcrypt" "$VERSION_LIBGCRYPT"   "tar.bz2"
+RET="$RET $?"
 
 for VALUE in $RET
 do
